function [velocity_field] = calculate_plane_flow_timestamps(XYZ, centers_deg, translate, rotate, view_dist, current_stamp, dist, nFlowFields, stimulus_duration)
% Author: Charlie S. Burlingham
% Last Updated: Nov 23, 2020
%
% Purpose: generates a single optic flow field for specified 
% ego-translation and rotation velocities, 3-d world (depth structure),
% timebase, and current time. Optic flow is generated at a fixed set of
% image locations ("centers_deg").
%
% The world is a plane, the observer is translating and rotating past it.
% The observer is always at the origin, and is looking down the Z axis.
% This  program takes a set of gratings and calculates the phase velocity of each
% one based on given translational and rotational components of optic flow
% 
% **** Inputs ****
% XYZ is the XYZ position (in m) of the 3-d world locations.
%
% centers_deg is the x,y positions of the optic flow vector tails (also grating centers) on the screen (in deg).
%
% translate tells you the observer's translation speed (Vx, Vy, Vz) in m/sec.
%
% rotate tells you the observer's rotation (Wx, Wy, Wz) in rad/s.
%
% view_dist is how far the actual observer is the from the monitor in meters.
%
% current_stamp is the flow field we are iterating on out of the total nFlowFields.
%
% dist is the initial distance of the simulated observer / camera from the the plane (m).
%
% stimulus_duration is the duration of the sequence in seconds (4 sec in our experiment)
%
% **** Outputs ****
% velocity_field is the optic flow field field - a matrix (2x[number of locations])
% reporting the x and y velocity at each image location in deg/s
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

% default inputs
if nargin==0
    dist = 12.5; % plane is 12.5 meters in depth away
    transSpeed = 1.5; % m/s
    heading = 0; % deg, horizontal heading angle
    translate = [transSpeed.*sind(heading) 0 transSpeed.*cosd(heading)]; % the observer is walking forward at 1.5 m/s, a brisk walk
    [XYZ, centers_deg] = make_dot_plane(.16, dist, [40 32], [0 0 0 0]); % make the 3d environment: an (initially) fronto-parallel plane composed of dots. The last arugment [0 0 0 0] is the exclusion region.
    rotate = deg2rad([0 2 0]); % rotation velocity in rad/s
    view_dist = .57; % focal length of projection
    nFlowFields = 10; % total number of flow fields in sequence
    stimulus_duration = 4; % stimulus duration in s
    current_stamp = 0; % current timestap from 0 to nFlowFields
end

% initialize empty matrix to hold velocities
velocity_field1 = zeros(size(centers_deg)); % number of patches, X and Y vel

% convert the positions in deg to m, because the other measurements are already in m.
centers_m = view_dist*tand(centers_deg);

% Time Stamp Constants
translate_calculation = (translate/nFlowFields) * stimulus_duration; % change translation to m/frame
rotate_calculation = ((rotate)/nFlowFields) * stimulus_duration; % change rotation to degrees/frame

% components of equation of a plane (we are rotating the plane itself)
d = dist - translate_calculation(3) * current_stamp;
a = tan(current_stamp * rotate_calculation(2));

% Solve for the inverse depth at the X,Y,Z position that projects to the
% desired x,y position, then use that to compute the flow vector at each
% location (distinct Z value for each of the gabors).
for i=1:length(XYZ)
    
    % Eqs. in this loop are from Heeger & Jepson 1993
    x = centers_m(1,i);
    y = centers_m(2,i);
    f = view_dist;
    
    % Note: for low rotation velocities, rotating a plane around its own
    % center as it translates forward approximates relative motion of the
    % environment during ego-motion along a circular path very closely.
    % That's what's implemented in this next line.
    XYZ(3,i) = ((-d*f) ./ ((a*x)-f)); % Note: another way of writing Z = aX + Z_0, with x = fX/Z subbed in for X, as in David Heeger's motion estimation pdf. 
    inv_depth = 1/XYZ(3,i); % p = 1/Z
    
    Pxy = inv_depth; 
    
    % matrices that convert 3d coordinates to 2d retinocentric coordinates
    A = [-f 0 x; 0 -f y];
    B = [(x*y)/f -(f+x^2/f) y; f+y^2/f -(x*y)/f -x];
    
    velocity_field1(1:2,i) = Pxy*A*translate_calculation'+B*rotate_calculation';
end

% Convert velocities in m/frame to deg/sec
velocity_field2 = (velocity_field1*nFlowFields)/stimulus_duration;
velocity_field = atand(velocity_field2/view_dist);

end