% Title: Rieger & Lawton (1985) Implementation 
%
% Last Updated: July 6, 2020
%
% Purpose: Estimates heading from instantaneous optic flow using the method 
% of Rieger & Lawton (1985) with one modification. Anisotropy-based 
% thresholding of difference vectors increased heading bias in our tests, 
% so I removed it from the estimator. This script generates sequences of 
% optic flow fields conveyed by the stimuli in Burlingham & Heeger (2020) 
% and subsequently estimates heading from these flow fields.
%
% Usage: run this script from /Rieger&Lawton/
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


%% initialize and flags

clear; clc; 

addpath(genpath(pwd));

plotMode = 0; %  If 0, will plot only the 1 fig from Fig S6. Only set to 1 if you put a debug point at the end of estimateHeadingRL.
saveFig = 0; % Save fig from SI

%% Set parameters to vary across loop

rotVels = [-2 -.8 0 .8 2];
%headingss = [-15:5:15]; %<- note uncomment to loop over headings instead of rotation velocities

%% loop

for hh = 1:length(rotVels)
% init params
stimulus = [];

%% Stimulus parameters
% circular path.
stimulus.plane_dist = [12.5 25]; % meters
stimulus.nPlanes = length(stimulus.plane_dist); % 2 planes 
stimulus.view_window = [40 32]; % dimensions of CRT monitor
stimulus.exclude = [-15 -2.25 15 2.25];  % in Deg. Don't place elements where the FOE will be.
stimulus.nFlowFields = 10; % frames of flow
stimulus.view_dist = .57; % meters, focal length
stimulus.duration = 4; % seconds

% main parameters one might want to vary ***
stimulus.dot_density = 1.*[.16 .16]; % dots/deg^2
transSpeed = 1.5; % m/s
rotVel = rotVels(hh); % deg/s
stimulus.rotate = [0 rotVel 0]; % just yaw rotation

stimulus.e = 3; % deg. ** Window size  threshold on euclidean distance.  see p. 2, Rieger & Lawton 1985

% loop over variable
stimulus.heading =  0; % headingss(hh); % deg.  <-  NOTES: change line to "stimulus.heading = headingss(hh)" if you want to loop through many headings instead of rotation velocities. If so, also set line 63 to: rotVel = 2; % deg/s
stimulus.translate = [transSpeed.*sind(stimulus.heading) 0 transSpeed.*cosd(stimulus.heading)];    % the observer is walking forward at 1.5 m/s, a brisk walk

iters = 40; % number of times to iterate (for different random samplings of dot positions)

headingEst = estimateHeadingRL(stimulus,plotMode,iters);

meanHeadingEst(hh,:) = nanmean(headingEst) % print mean heading estimate 
stdHeadingEst(hh,:) = nanstd(headingEst) % print standard deviation of heading estimate

end


%% plot results
figure(1); hold on; 
errorbar(rotVels,meanHeadingEst(:,1),stdHeadingEst(:,1),'o-'); xlabel('Rotation Velocity (/s)');
ylabel('Heading (deg)');
title('Rieger & Lawton 1985');
box off;
grid on;
hold on; line([-2.2 2.2],[0 0],'LineWidth',1,'Color',[0 0 0]);
%axis equal
xlim([-2.2 2.2]); %ylim([-20 20])
legend('Estimate, R&L','True heading ')

if saveFig
savefig('changingHeading_bias')
print('changingHeading_bias','-dpng')
end
