function [wsa] = OFAnalysis(dirname)
%   OFAnalysis.m
%      usage: OFAnalysis(dirname)
%         by: Charlie S. Burlingham
%       date: 11/23/2020
%    purpose: Fits psychometric function to choice data using Psignifit
%             (see Schutt et al, Vis Research 2016)
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


% navigate to directory with mgl-outputted data files (.mat)
contents = dir(dirname);

matches = strfind({contents.name},'stim');

contentscell = struct2cell(contents);
filenames = contentscell(1,:);

cd(dirname);
indices = [];

for i = 1:length(matches)
    if matches{i} == 8
        indices = horzcat(indices, i);
    end
end

for i = 1:length(indices)
    data(i) = load(sprintf('%s',filenames{indices(i)}));
    datacell{i} = struct2cell(data(i));
    datacell{i} = datacell{1,i}{1,1};
end

% Initialize response arrays pooled across runs
responseVecLlo = [];  responseVecLhi = []; responseVecRlo = []; responseVecRhi = []; responseVecPT = [];
% Initialize heading arrays pooled across runs
headingVecLlo = []; headingVecLhi = []; headingVecRlo = [];  headingVecRhi = []; headingVecPT = [];
% pool data across runs within subject
for i = 1:length(indices)
    responseVecLlo = [responseVecLlo datacell{1,i}.responseLlo];
    responseVecLhi = [responseVecLhi datacell{1,i}.responseLhi];
    responseVecRlo = [responseVecRlo datacell{1,i}.responseRlo];
    responseVecRhi = [responseVecRhi datacell{1,i}.responseRhi];
    responseVecPT = [responseVecPT datacell{1,i}.responsePT];
    
    headingVecLlo = [headingVecLlo datacell{1,i}.headingLlo];
    headingVecLhi = [headingVecLhi datacell{1,i}.headingLhi];
    headingVecRlo = [headingVecRlo datacell{1,i}.headingRlo];
    headingVecRhi = [headingVecRhi datacell{1,i}.headingRhi];
    headingVecPT = [headingVecPT datacell{1,i}.headingPT];
end

% Now pad with NaNs if some arrays are missing values

% get max length
lengthsArray = [length(responseVecLlo); length(responseVecRlo); length(responseVecLhi); length(responseVecRhi); length(responseVecPT);];

maxLength = max(lengthsArray);

numNaNstoPadWith = maxLength - length(responseVecLlo);
responseVecLlo = cat(2,responseVecLlo,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(responseVecLhi);
responseVecLhi = cat(2,responseVecLhi,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(responseVecRlo);
responseVecRlo = cat(2,responseVecRlo,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(responseVecRhi);
responseVecRhi = cat(2,responseVecRhi,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(responseVecPT);
responseVecPT = cat(2,responseVecPT,NaN(numNaNstoPadWith,1)');

numNaNstoPadWith = maxLength - length(headingVecLlo);
headingVecLlo = cat(2,headingVecLlo,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(headingVecLhi);
headingVecLhi = cat(2,headingVecLhi,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(headingVecRlo);
headingVecRlo = cat(2,headingVecRlo,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(headingVecRhi);
headingVecRhi = cat(2,headingVecRhi,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(headingVecPT);
headingVecPT = cat(2,headingVecPT,NaN(numNaNstoPadWith,1)');

% put all NaN padded data in matrices, conditions x data-by-trial
responseMat = [responseVecRhi; responseVecRlo; responseVecPT; responseVecLlo; responseVecLhi];
headingMat = [headingVecRhi; headingVecRlo; headingVecPT; headingVecLlo; headingVecLhi];

% Now sort the heading matrix along the horizontal axis, save, indices, and
% then sort the response matrix with the same indices.
[sortedHeading indices] = sort(headingMat,2);

sortedResponses = NaN(size(responseMat,1),size(responseMat,2));
for ii = 1:size(responseMat)
    currentRowResponseMat = responseMat(ii,:);
    sortedResponses(ii,:) = currentRowResponseMat(indices(ii,:));
end

%%%% NOTE: may have to switch 1 and 0 below for datafiles from EM vs. (TV)PM conditions, because of a minor recoding error
% Recode responses from 1 and 2 keys to binary digits (so I can get percent rightward resp in each bin out of 100%
sortedResponses(find(sortedResponses==1)) = 1; % recode the responses backwards bc of the encoding from left to right I do in expt script
sortedResponses(find(sortedResponses==2)) = 0;

% save out resp and stimLevel data for Stan fitting
save('stimValforStan','sortedHeading')
save('respforStan','sortedResponses')

%%
% Optionally- re-structure data for analysis with psignifit (as sanity
% check that my fitting or RStan fitting is working.

% fit and plot psychometric functions, depending on how much data there is.
% also get confidence intervals on all the parameter fits
PsignifitParams = []; ninetyfivepcConfintBias = []; ninetyfivepcConfintVar = []; devianceVec = [];marginalMu = {}; xVals = {};
for ii = 1:size(sortedHeading,1)
    options.sigmoidName  = 'norm';
    options.expType = 'equalAsymptote';
    
    inputForPsignifit = [sortedHeading(ii,:); sortedResponses(ii,:); ones(length(sortedResponses),1)']';
    nanLoc = find(isnan(inputForPsignifit(:,2)));
    inputForPsignifit(nanLoc,:) = [];
    
    
    fitOutput = psignifit(inputForPsignifit,options); 
    
    threshold(ii) = getThreshold(fitOutput,0.75)-getThreshold(fitOutput,0.50); % should be very close to sigma, aka the second row of Psignifitparams / 2*1.96
    
    plotOptions.xLabel         = 'Heading Direction in Visual Degrees';     % xLabel
    plotOptions.yLabel         = '% Rightward Responses';    % yLabel
    plotOptions.CIthresh       = true;
    if ii < 3 % To plot the left, PT, and right curves in independant colours!
        plotOptions.dataColor      = [0 0 1];  % color of the data
    elseif ii == 3
        plotOptions.dataColor      = [0 1 0];  % color of the data
    elseif ii>3
        plotOptions.dataColor      = [1 0 0];  % color of the data
    end
    plotOptions.lineColor = plotOptions.dataColor;
    
    figure(1)
    [hline1] = plotPsych(fitOutput,plotOptions);
    hold on
    
    PsignifitParams = [PsignifitParams; fitOutput.Fit(1) fitOutput.Fit(2)];
    
    marginalMu = [marginalMu fitOutput.marginals{1, 1}];
    
    xVals = [xVals fitOutput.X1D{1, 1}];
    
    ninetyfivepcConfintBias = [ninetyfivepcConfintBias; fitOutput.conf_Intervals(1,:,1)];
    ninetyfivepcConfintVar = [ninetyfivepcConfintVar; fitOutput.conf_Intervals(2,:,1)];
    
    % The order of reported parameters is
    % [threshold,width,lambda,gamma,eta]
    
    % optional- get the deviance of the model from the real data
    [devianceResid devianceVal] = getDeviance(fitOutput);
    devianceVec = [devianceVec devianceVal];
    
    % Optional if you want to plot deviance of model from real data
    %figure;plotsModelfit(fitOutput);
end
save('PsignifitParams','PsignifitParams')
save('ninetyfivepcConfintBias','ninetyfivepcConfintBias')
save('ninetyfivepcConfintVar','ninetyfivepcConfintVar')
save('devianceVec','devianceVec')
save('marginalMu','marginalMu')
save('xVals','xVals')
save('threshold','threshold')


figure(1)
%savefig('curvesPlot')

% Optionally plot bias with 95% confidence intervals
figure
errorbar(1:size(PsignifitParams,1),PsignifitParams(:,1),PsignifitParams(:,1)-ninetyfivepcConfintBias(:,1),ninetyfivepcConfintBias(:,2)-PsignifitParams(:,1),'rs')
xlabel('Rotation Condition','Fontsize',24)
ylabel('Fitted Mu (visual degrees)','Fontsize',24)
title('Bias Plot Psignifit w/ 95% CIs','Fontsize',30)
set(gca,'XTickLabel',{'Left Fast', 'Left Slow', 'Pure Translation', 'Right Slow', 'Right Fast'},'Fontsize',12)
set(gca,'XTick',1:size(sortedHeading,1))
hline(0)
savefig('biasPlotPsignifitwCIs')

% Optionally plot variance with 95% confidence intervals
figure
errorbar(1:size(PsignifitParams,1),PsignifitParams(:,2)./(2*1.98),PsignifitParams(:,2)./(2*1.98)-ninetyfivepcConfintVar(:,1)./(2*1.98),ninetyfivepcConfintVar(:,2)./(2*1.98)-PsignifitParams(:,2)./(2*1.98),'ko')
xlabel('Rotation Condition','Fontsize',24)
ylabel('Fitted Sigma (visual degrees)','Fontsize',24)
title('Variance Plot Psignifit w/ 95% CIs','Fontsize',30)
set(gca,'XTickLabel',{'Left Fast', 'Left Slow', 'Pure Translation', 'Right Slow', 'Right Fast'},'Fontsize',12)
set(gca,'XTick',1:size(sortedHeading,1))
hline(0)
savefig('varPlotPsignifitwCIs')
