function [wsa] = OFAnalysis_controlExpts(dirname)
%   OFAnalysis_controlExpts.m
%      usage: OFAnalysis_controlExpts(dirname)
%         by: Charlie S. Burlingham
%       date: 11/23/2020
%    purpose: Fits psychometric function to choice data using Psignifit
%             (see Schutt et al, Vis Research 2016).
%             -> especially for control experiments where we only used
%                +2 deg/s rotations
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

close all; %rng('shuffle')

contents = dir(dirname);

matches = strfind({contents.name},'stim');

contentscell = struct2cell(contents);
filenames = contentscell(1,:);

cd(dirname);
indices = [];

for i = 1:length(matches)
    if matches{i} == 8
        indices = horzcat(indices, i);
    end
end

for i = 1:length(indices)
    data(i) = load(sprintf('%s',filenames{indices(i)}));
    datacell{i} = struct2cell(data(i));
    datacell{i} = datacell{1,i}{1,1};
end


% Initialize response arrays pooled across runs
responseVecLhi = []; responseVecRhi = []; responseVecPT = [];
% Initialize heading arrays pooled across runs
headingVecLhi = []; headingVecRhi = []; headingVecPT = [];
% pool data across runs within subject
for i = 1:length(indices)
    if isfield(datacell{1,i}, 'responseLhi')
        responseVecLhi = [responseVecLhi datacell{1,i}.responseLhi];
    end
    if isfield(datacell{1,i}, 'responseRhi')
        responseVecRhi = [responseVecRhi datacell{1,i}.responseRhi];
    end
    if isfield(datacell{1,i}, 'responsePT') 
        responseVecPT = [responseVecPT datacell{1,i}.responsePT];
    end
    
    if isfield(datacell{1,i}, 'headingLhi')
        headingVecLhi = [headingVecLhi datacell{1,i}.headingLhi];
    end
    if isfield(datacell{1,i}, 'headingRhi')
        headingVecRhi = [headingVecRhi datacell{1,i}.headingRhi];
    end
    if isfield(datacell{1,i}, 'headingPT')
        headingVecPT = [headingVecPT datacell{1,i}.headingPT];
    end
end

% Now pad with NaNs if some arrays are missing values

% get max length
lengthsArray = [length(responseVecLhi); length(responseVecRhi); length(responseVecPT);];

maxLength = max(lengthsArray);

numNaNstoPadWith = maxLength - length(responseVecLhi);
responseVecLhi = cat(2,responseVecLhi,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(responseVecRhi);
responseVecRhi = cat(2,responseVecRhi,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(responseVecPT);
responseVecPT = cat(2,responseVecPT,NaN(numNaNstoPadWith,1)');

numNaNstoPadWith = maxLength - length(headingVecLhi);
headingVecLhi = cat(2,headingVecLhi,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(headingVecRhi);
headingVecRhi = cat(2,headingVecRhi,NaN(numNaNstoPadWith,1)');
numNaNstoPadWith = maxLength - length(headingVecPT);
headingVecPT = cat(2,headingVecPT,NaN(numNaNstoPadWith,1)');

% put all NaN padded data in matrices, conditions x data-by-trial
responseMat = [responseVecRhi; responseVecPT; responseVecLhi];
headingMat = [headingVecRhi; headingVecPT; headingVecLhi];

% Now sort the heading matrix along the horizontal axis, save, indices, and
% then sort the response matrix with the same indices.
[sortedHeading indices] = sort(headingMat,2);

sortedResponses = NaN(size(responseMat,1),size(responseMat,2));
for ii = 1:size(responseMat)
    currentRowResponseMat = responseMat(ii,:);
    sortedResponses(ii,:) = currentRowResponseMat(indices(ii,:));
end

% Recode responses from 1 and 2 keys to binary digits (so I can get percent rightward resp in each bin out of 100%
sortedResponses(find(sortedResponses==1)) = 0;
sortedResponses(find(sortedResponses==2)) = 1;

% save out resp and stimLevel data for Stan fitting
save('stimValforStan','sortedHeading')
save('respforStan','sortedResponses')

%% Fit pscyhometric functions

PsignifitParams = []; ninetyfivepcConfintBias = []; ninetyfivepcConfintVar = []; devianceVec = [];marginalMu = {}; xVals = {};
for ii = 1:size(sortedHeading,1)
    inputForPsignifit = [sortedHeading(ii,:); sortedResponses(ii,:); ones(length(sortedResponses),1)']';
    options.sigmoidName  = 'norm';
    options.expType = 'equalAsymptote';
    
    % Important! trim all rows with NaNs (missing responses) before fitting
    nanLoc = find(isnan(inputForPsignifit(:,2)));
    inputForPsignifit(nanLoc,:) = [];
    
    fitOutput = psignifit(inputForPsignifit,options);
    threshold(ii) = getThreshold(fitOutput,0.75)-getThreshold(fitOutput,0.50); % should be very close to sigma, aka the second row of Psignifitparams / 2*1.96
    
    plotOptions.xLabel         = 'Heading Direction in Visual Degrees';     % xLabel
    plotOptions.yLabel         = '% Rightward Responses';    % yLabel
    plotOptions.CIthresh       = true;
    if ii ==1 % To plot the left, PT, and right curves in independant colours!
        plotOptions.dataColor      = [0 0 1];  % color of the data
    elseif ii == 2
        plotOptions.dataColor      = [0 1 0];  % color of the data
    elseif ii ==3
        plotOptions.dataColor      = [1 0 0];  % color of the data
    end
    plotOptions.lineColor = plotOptions.dataColor;
    
    figure(1)
    [hline1] = plotPsych(fitOutput,plotOptions);
    hold on
    
    PsignifitParams = [PsignifitParams; fitOutput.Fit(1) fitOutput.Fit(2)];
    
    marginalMu = [marginalMu fitOutput.marginals{1, 1}];
    
    xVals = [xVals fitOutput.X1D{1, 1}];
    
    ninetyfivepcConfintBias = [ninetyfivepcConfintBias; fitOutput.conf_Intervals(1,:,1)];
    ninetyfivepcConfintVar = [ninetyfivepcConfintVar; fitOutput.conf_Intervals(2,:,1)];
    % The order of reported parameters is
    % [threshold,width,lambda,gamma,eta]
    
    % optional- get the deviance of the model from the real data
    [devianceResid devianceVal] = getDeviance(fitOutput);
    devianceVec = [devianceVec devianceVal];
    
    % Optional if you want to plot deviance of model from real data
    %figure;plotsModelfit(fitOutput);
end

save('PsignifitParams','PsignifitParams')
save('ninetyfivepcConfintBias','ninetyfivepcConfintBias')
save('ninetyfivepcConfintVar','ninetyfivepcConfintVar')
save('devianceVec','devianceVec')
save('marginalMu','marginalMu')
save('xVals','xVals')
save('threshold','threshold')

figure(1)
savefig('curvesPlot')

% Optionally plot bias with 95% confidence intervals
figure
errorbar(1:size(PsignifitParams,1),PsignifitParams(:,1),PsignifitParams(:,1)-ninetyfivepcConfintBias(:,1),ninetyfivepcConfintBias(:,2)-PsignifitParams(:,1),'ko')
xlabel('Rotation Condition','Fontsize',24)
ylabel('Fitted Mu (visual degrees)','Fontsize',24)
title('Bias Plot Psignifit w/ 95% CIs','Fontsize',30)
set(gca,'XTickLabel',{'Left 2/s', 'Pure Translation','Right 2 /s'},'Fontsize',12)
set(gca,'XTick',1:size(sortedHeading,1))
hline(0)
savefig('biasPlotPsignifitwCIs')

% Optionally plot variance with 95% confidence intervals
figure
stdPar = PsignifitParams(:,2)./(2.*1.96); CIpar = ninetyfivepcConfintVar./(2.*1.96);
errorbar(1:size(PsignifitParams,1),stdPar,stdPar-CIpar(:,1),CIpar(:,2)-stdPar,'ko')
xlabel('Rotation Condition','Fontsize',24)
ylabel('Fitted Sigma (visual degrees)','Fontsize',24)
title('Variance Plot Psignifit w/ 95% CIs','Fontsize',30)
set(gca,'XTickLabel',{'Left 2/s', 'Pure Translation','Right 2 /s'},'Fontsize',12)
set(gca,'XTick',1:size(sortedHeading,1))
hline(0)
savefig('varPlotPsignifitwCIs')

