% runEyeAnal.m
%      usage: runEyeAnal(stimfile,plotMode)
%         by: Charlie Burlingham
%       date: 04/19/2018
%    purpose: plot eye data from tvof experiment
%             Run on a single stimfile from one subject. plotMode = 1 turns 
%             on plotting.
%

function retval = runEyeAnal(stimfile)

% default return argument
retval = [];

% Load the task data
stimfile = load(stimfile);

% get the eyetracker edf filename
if isfield(stimfile.myscreen.eyetracker,'datafilename')
    eyeTrackerFilename = stimfile.myscreen.eyetracker.datafilename;
else
    disp(sprintf('(getTaskEyeTraces) No associated eyetracker datafilename found in myscreen'));
    return
end

% load the eye tracking data
edf = mglEyelinkEDFRead(eyeTrackerFilename);

% remove blinks
blinkWindow = ceil(0.5*edf.samplerate); % default
%blinkWindow = ceil(removeBlink*edf.samplerate);
for Bn = 1:numel(edf.blinks.startTime)
    blinks = (edf.gaze.time >= edf.blinks.startTime(Bn)-blinkWindow & ...
        edf.gaze.time <= edf.blinks.endTime(Bn)+blinkWindow);
    edf.gaze.x(blinks) = NaN;
    edf.gaze.y(blinks) = NaN;
    edf.gaze.pupil(blinks) = NaN;
end

% interpolate linearly over blinks
edf.gaze.x(isnan(edf.gaze.x)) = interp1(find(~isnan(edf.gaze.x)), ...
    edf.gaze.x(~isnan(edf.gaze.x)), find(isnan(edf.gaze.x)), 'linear');
edf.gaze.y(isnan(edf.gaze.y)) = interp1(find(~isnan(edf.gaze.y)), ...
    edf.gaze.y(~isnan(edf.gaze.y)), find(isnan(edf.gaze.y)), 'linear');

% convert to degrees of visual angle
imageWidth = 55;
imageHeight = 32;
screenWidth = 1152;
screenHeight = 870;
edf.gaze.x = imageWidth*(edf.gaze.x - screenWidth/2)/screenWidth;
edf.gaze.y = imageHeight*(edf.gaze.y-screenHeight/2)/screenHeight;

%% now detect saccades

saccVelThreshold = 100; % 100 per second or above only are defined as saccades
eyedat = saccadeDetect(edf,stimfile,saccVelThreshold); 
saccadeTimes = eyedat.saccades(:,1:2);

%%

% recode trial types by EM condition - simulated or real (L or R) (0, 1, or 2)
recodedTrialTypesEM = stimfile.stimulus.trialType;
%recodedTrialTypesEMsimulated = find(recodedTrialTypesEM~=4&recodedTrialTypesEM~=9&recodedTrialTypesEM~=13&recodedTrialTypesEM~=5&recodedTrialTypesEM~=10&recodedTrialTypesEM~=14); % recode all other trials
recodedTrialTypesEMrLslow = find(recodedTrialTypesEM==1); % recode  rotation left slow trials
recodedTrialTypesEMrLfast = find(recodedTrialTypesEM==2); % recode  rotation left fast trials

recodedTrialTypesEMrRslow = find(recodedTrialTypesEM==3); % recode  rotation right slow trials
recodedTrialTypesEMrRfast = find(recodedTrialTypesEM==4); % recode  rotation right fast trials
recodedTrialTypesEMPT = find(recodedTrialTypesEM==5); % recode  rotation right fast trials

% find the first trial
istrial = []; firstSample = 1;
while isempty(istrial)
    istrial = strfind(edf.messages(firstSample).message, 'BEGIN TRIAL 1');
    firstSample = firstSample + 1;
end
nMessages = length(edf.messages) - firstSample;
trialLengthSeconds = 5;
nSamplesPerTrial = edf.samplerate * trialLengthSeconds;


shiftAmt = 0;

xPosPT = NaN(length(recodedTrialTypesEMPT), nSamplesPerTrial); yPosPT = NaN(length(recodedTrialTypesEMPT), nSamplesPerTrial);
n = 1; tmp = [];numSaccsPT = zeros(length(recodedTrialTypesEMPT),1);
% pure translation trials
for ii = 1:length(recodedTrialTypesEMPT)
    for iSample=firstSample:firstSample + nMessages-1
        if strfind(edf.messages(iSample).message, sprintf('TRIAL %2i %2i',recodedTrialTypesEMPT(ii)',recodedTrialTypesEMPT(ii)')) % find the data in the edf that match those trials where the eye movement was left. Have to repeat the number string to disambiguate 21 from 2 or 22 (for example)
            tmp = abs(edf.gaze.time - edf.messages(iSample).time);
            [minVal, startInd] = min(tmp);
            tmp = abs(edf.gaze.time - edf.messages(iSample+1).time);
            [minVal, endInd] = min(tmp);
            xPosPT(n,:) = edf.gaze.x(startInd+shiftAmt:startInd+nSamplesPerTrial+shiftAmt-1);
            yPosPT(n,:) = edf.gaze.y(startInd+shiftAmt:startInd+nSamplesPerTrial+shiftAmt-1);
            
            numSaccsPT(n) = numSaccsPT(n) + sum(saccadeTimes(:,1) > startInd & saccadeTimes(:,2) < startInd+nSamplesPerTrial);
            n = n+1;
        end
    end
end


retval.avgSaccs{3} = numSaccsPT;
retval.eyeSignal{3} = [nanmean(xPosPT); nanmean(yPosPT)]';


xPosLslow = NaN(length(recodedTrialTypesEMrRfast), nSamplesPerTrial); yPosLslow = NaN(length(recodedTrialTypesEMrLslow), nSamplesPerTrial);
n = 1; tmp = []; numSaccsLslow = zeros(length(recodedTrialTypesEMrLslow),1);
% left slow trials
for ii = 1:length(recodedTrialTypesEMrLslow)
    for iSample=firstSample:firstSample + nMessages-1
        if strfind(edf.messages(iSample).message, sprintf('TRIAL %2i %2i',recodedTrialTypesEMrLslow(ii)',recodedTrialTypesEMrLslow(ii)')) % find the data in the edf that match those trials where the eye movement was left. Have to repeat the number string to disambiguate 21 from 2 or 22 (for example)
            tmp = abs(edf.gaze.time - edf.messages(iSample).time);
            [minVal, startInd] = min(tmp);
            tmp = abs(edf.gaze.time - edf.messages(iSample+1).time);
            [minVal, endInd] = min(tmp);
            xPosLslow(n,:) = edf.gaze.x(startInd+shiftAmt:startInd+nSamplesPerTrial+shiftAmt-1);
            yPosLslow(n,:) = edf.gaze.y(startInd+shiftAmt:startInd+nSamplesPerTrial+shiftAmt-1);
            
            numSaccsLslow(n) = numSaccsLslow(n) + sum(saccadeTimes(:,1) > startInd & saccadeTimes(:,2) < startInd+nSamplesPerTrial);
            
            n = n+1;
            
        end
    end
end

retval.avgSaccs{2} = numSaccsLslow;
retval.eyeSignal{2} = [nanmean(xPosLslow); nanmean(yPosLslow)]';


xPosRslow = NaN(length(recodedTrialTypesEMrRslow), nSamplesPerTrial); yPosRslow = NaN(length(recodedTrialTypesEMrRslow), nSamplesPerTrial);
n = 1; tmp = []; numSaccsRslow = zeros(length(recodedTrialTypesEMrRslow),1);
% right slow
for ii = 1:length(recodedTrialTypesEMrRslow)
    for iSample=firstSample:firstSample + nMessages -1
        if strfind(edf.messages(iSample).message, sprintf('TRIAL %2i %2i',recodedTrialTypesEMrRslow(ii)',recodedTrialTypesEMrRslow(ii)')) % find the data in the edf that match those trials where the eye movement was right. Have to repeat the number string to disambiguate 21 from 2 or 22
            tmp = abs(edf.gaze.time - edf.messages(iSample).time);
            [minVal, startInd] = min(tmp);
            tmp = abs(edf.gaze.time - edf.messages(iSample+1).time);
            [minVal, endInd] = min(tmp);
            xPosRslow(n,:) = edf.gaze.x(startInd+shiftAmt:startInd+nSamplesPerTrial+shiftAmt-1);
            yPosRslow(n,:) = edf.gaze.y(startInd+shiftAmt:startInd+nSamplesPerTrial+shiftAmt-1);
            
            numSaccsRslow(n) = numSaccsRslow(n) + sum(saccadeTimes(:,1) > startInd & saccadeTimes(:,2) < startInd+nSamplesPerTrial);
            
            n = n+1;
        end
    end
end

retval.avgSaccs{4} = numSaccsRslow;
retval.eyeSignal{4} = [nanmean(xPosRslow); nanmean(yPosRslow)]';


xPosLfast = NaN(length(recodedTrialTypesEMrLfast), nSamplesPerTrial); yPosLfast = NaN(length(recodedTrialTypesEMrLfast), nSamplesPerTrial);
n = 1;tmp = []; numSaccsLfast = zeros(length(recodedTrialTypesEMrLfast),1);
% left fast
for ii = 1:length(recodedTrialTypesEMrLfast)
    for iSample=firstSample:firstSample + nMessages -1
        if strfind(edf.messages(iSample).message, sprintf('TRIAL %2i %2i',recodedTrialTypesEMrLfast(ii)',recodedTrialTypesEMrLfast(ii)')) % find the data in the edf that match those trials where the eye movement was simulated. Have to repeat the number string to disambiguate 21 from 2 or 22
            tmp = abs(edf.gaze.time - edf.messages(iSample).time);
            [minVal, startInd] = min(tmp);
            tmp = abs(edf.gaze.time - edf.messages(iSample+1).time);
            [minVal, endInd] = min(tmp);
            xPosLfast(n,:) = edf.gaze.x(startInd+500:startInd+nSamplesPerTrial+500-1);
            yPosLfast(n,:) = edf.gaze.y(startInd+500:startInd+nSamplesPerTrial+500-1);
            
            numSaccsLfast(n) = numSaccsLfast(n) + sum(saccadeTimes(:,1) > startInd & saccadeTimes(:,2) < startInd+nSamplesPerTrial);
            
            n = n+1;
        end
    end
end

retval.avgSaccs{1} = numSaccsLfast;
retval.eyeSignal{1} = [nanmean(xPosLfast); nanmean(yPosLfast)]';


xPosRfast = NaN(length(recodedTrialTypesEMrRfast), nSamplesPerTrial); yPosRfast = NaN(length(recodedTrialTypesEMrRfast), nSamplesPerTrial);
n = 1; tmp = []; numSaccsRfast = zeros(length(recodedTrialTypesEMrRfast),1);
% right fast
for ii = 1:length(recodedTrialTypesEMrRfast)
    for iSample = firstSample:firstSample + nMessages -1
        if strfind(edf.messages(iSample).message, sprintf('TRIAL %2i %2i',recodedTrialTypesEMrRfast(ii)',recodedTrialTypesEMrRfast(ii)')) % find the data in the edf that match those trials where the eye movement was simulated. Have to repeat the number string to disambiguate 21 from 2 or 22
            tmp = abs(edf.gaze.time - edf.messages(iSample).time);
            [minVal, startInd] = min(tmp);
            tmp = abs(edf.gaze.time - edf.messages(iSample+1).time);
            [minVal, endInd] = min(tmp);
            xPosRfast(n,:) = edf.gaze.x(startInd+500:startInd+nSamplesPerTrial+500-1);
            yPosRfast(n,:) = edf.gaze.y(startInd+500:startInd+nSamplesPerTrial+500-1);
            
            numSaccsRfast(n) = numSaccsRfast(n) + sum(saccadeTimes(:,1) > startInd & saccadeTimes(:,2) < startInd+nSamplesPerTrial);
            
            n = n+1;
        end
    end
end

retval.avgSaccs{5} = numSaccsRfast;
retval.eyeSignal{5} = [nanmean(xPosRfast); nanmean(yPosRfast)]';
