function eyedat = saccadeDetect(edf,stimfile,velThreshold)
% Author: Charlie Burlingham - modded from code by Stuart Jackson  - https://github.com/sj971/neurosci-saccade-detection/blob/master/analyzeEyeData.m
% Date: apr 22, 2018
% function eyeDat = saccadeDetect(edf,stimfile,plotMode)
% for use with mgl stimfiles and eyelink data
% Purpose: finds saccades in eye data. use after interpolating out blinks

%% 1. Set appropriate variables

% common parameters
nTrials = stimfile.task{1, 1}.numTrials;
sample_window = 1/edf.samplerate; %2ms i.e., 500Hz tracking
nEdgeSamples = 50; %number of samples to discard either side of blink
if ieNotDefined('velThreshold'); velThreshold = 30; end %threshold velocity for saccades (deg/s)
tAcceleration = 8000; %threshold acceleration for saccades (deg/s^2)
tDist = 0.5; %threshold (Euclidean) distance for saccades (deg)

xPos = edf.gaze.x;
yPos = edf.gaze.y;

%% calculate velocity/acceleration using 5-sample window. 
% See Engbert and Kliegl, 2003. Denominator accounts for the
% six sample 'differences' used in numerator (i.e., n-2 to
% n+2 = 4 samples, n-1 to n+1 = 2 samples).
xVel = zeros(size(xPos)); yVel = zeros(size(yPos));
for ii = 3:(size(xPos, 2) - 2) % 2 additional samples chopped off either end (see ~line 230 above)
    xVel(ii) = (xPos(ii + 2) + xPos(ii + 1) - xPos(ii - 1) - xPos(ii - 2))/(6*sample_window);
    yVel(ii) = (yPos(ii + 2) + yPos(ii + 1) - yPos(ii - 1) - yPos(ii - 2))/(6*sample_window);
end;
euclidVel = sqrt((xVel.*xVel) + (yVel.*yVel));

xAcc = zeros(size(xPos)); yAcc = zeros(size(yPos));
for ii = 3:(size(xVel, 2) - 2) % 2 additional samples chopped off either end (see ~line 230 above)
    xAcc(ii) = (xVel(ii + 2) + xVel(ii + 1) - xVel(ii - 1) - xVel(ii - 2))/(6*sample_window);
    yAcc(ii) = (yVel(ii + 2) + yVel(ii + 1) - yVel(ii - 1) - yVel(ii - 2))/(6*sample_window);
end;
euclidAcc = sqrt((xAcc.*xAcc) + (yAcc.*yAcc));

% save to post-processed eye data structure (pupil
% variables may now contain more samples than the others)
eyedat.xPos = xPos;
eyedat.yPos = yPos;
eyedat.xVel = xVel;
eyedat.yVel = yVel;
eyedat.xAcc = xAcc;
eyedat.yAcc = yAcc;
eyedat.euclidAcc = euclidAcc;
eyedat.euclidVel = euclidVel;

%%%%%%%%%%%%%%%%%%%%%%
% saccade detection
eyedat.saccades = [];
candidates = find(euclidVel > velThreshold);
if candidates
    
    % check for multiple candidate saccades in single
    % trial, using threshold parameters defined at top
    % (see Engbert & Kliegl papers, and Eyelink manual)
    saccades = [];
    diffCandidates = diff(candidates);
    breaks = [0 find(diffCandidates > 1) size(candidates, 2)];
    for jj = 1:(size(breaks, 2) - 1)
        
        % find individual candidate saccades
        saccade = [candidates(breaks(jj) + 1) candidates(breaks(jj + 1))];
        
        % exceeds acceleration threshold?
        peakAcceleration = max(euclidAcc(saccade(1):saccade(2)));
        if peakAcceleration > tAcceleration
            
            % exceeds amplitude threshold?
            xDist = xPos(saccade(2)) - xPos(saccade(1));
            yDist = yPos(saccade(2)) - yPos(saccade(1));
            euclidDist = sqrt((xDist*xDist) + (yDist*yDist));
            if euclidDist > tDist
                
                % store saccade info
                peakVelocity = max(euclidVel(saccade(1):saccade(2)));
                saccades = [saccades; saccade xDist yDist euclidDist peakVelocity];
            end
            
        end
        
    end
    
    % save saccade data
    eyedat.saccades = saccades;
    
end

end