function [stimulus] = OFexpt_EM(varargin)
%      title: OFexpt_EM.m
%      usage: run with external monitor and install MGL toolbox first
%         by: Charlie S. Burlingham 
%    updated: November 23, 2020
%
%    purpose: generates envelope motion stimulus video simulating travel along a circular path
%
%             Trial Types:
%             1) Envelope Motion, Simulated Eye Movement, Rotation Left, 0.8 /s 
%             2) Envelope Motion, Simulated Eye Movement, Rotation Left, 2 /s 
%             3) Envelope Motion, Simulated Eye Movement, Rotation Right 0.8 /s 
%             4) Envelope Motion, Simulated Eye Movement, Rotation Right, 2 /s 
%             5) Envelope Motion, Pure Translation
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

sca; mglClose; mglFlush;

getArgs(varargin,[],'verbose=1');

global MGL

%-----init some defaults for variables we can set in the function call ----------
if ieNotDefined('feedback'), feedback = 1; end % tone feedback and debug text or nah?
if ieNotDefined('threshStair0a'), threshStair0a = 20; end % start at limits of -20 and 20, so you have some data for the asymptotes of psychometric curve
if ieNotDefined('threshStair0b'), threshStair0b = 20; end
if ieNotDefined('threshStair1a'), threshStair1a = 20; end
if ieNotDefined('threshStair1b'), threshStair1b = 20; end
if ieNotDefined('threshStair2a'), threshStair2a = 20; end
if ieNotDefined('threshStair2b'), threshStair2b = 20; end
if ieNotDefined('stepsize'), stepsize = 4; end  %- this is the same stepsize as what's in Davis's/Ionatan's code
if ieNotDefined('whichSetup'), whichSetup = 1; end % 0: my desk (LCD) or 1: in testing room (CRT - 1280x1024, 60fps)

%-----init the task----------
mglSetParam('mustSetSID',false,2); % don't require subject ID database
myscreen.background = [127 127 127]; % set background color to grey
myscreen.autoCloseScreen = 0;
myscreen.saveData = 1; % autosave data
myscreen = initScreen(myscreen);

task{1}.waitForBacktick = 1;
task{1}.seglen = [4 1]; % two segments: 1st displays stimulus (2sec), 2th is response interval (1sec)
task{1}.getResponse = [0 1];
task{1}.random = 1;
task{1}.numTrials = 60; % must be divisible by 6 (num of trial conditions) so trial permutation works
% 60 trials * 5 seconds = 250 sec or 5 minutes

for phaseNum = 1:length(task)
    [task{phaseNum} myscreen] = initTask(task{phaseNum},myscreen,@startSegmentCallback,@screenUpdateCallback,@responseCallback);
end

%-----init the stimulus----------
global stimulus;
myscreen = initStimulus('stimulus',myscreen);
stimulus.feedback = feedback; % feedback on or off ? determines tone feedback and debug output
stimulus.whichSetup = whichSetup;
stimulus = myInitStimulus(stimulus,myscreen,task);

%-----Set up trial condition permutation structure
stimulus.n_conditions = 5;

stimulus.trialType = [];
for ii = 1:stimulus.n_conditions
    stimulus.trialType = [stimulus.trialType ii*ones((task{1}.numTrials/stimulus.n_conditions),1)']; % populate vector with equal proportions of 0's, 1's, 2's, etc., which represent ALL trial conditions
end

stimulus.trialType = stimulus.trialType(randperm(task{1}.numTrials));   % randomly permutate order to randomize trial types in the run

%-----Screen Landmarks, Counters, and Stimulus Duration------------

stimulus.trialCounter = 0;

stimulus_duration               = task{1, 1}.seglen(1);                                    % secs
if stimulus.whichSetup == 1
    frame_rate                      = 75;                                   % Screen frame rate (hz) of the CRT
else
    frame_rate                      = 60;                                  % Screen frame rate (hz)
end
stimulus.frame_rate =frame_rate;
stimulus.mv_length              = ceil(stimulus_duration*frame_rate);   % length of stimulus movie

%-----Staircase Settings------------
% Init FOUR 1 down 1 up staircase with a stepsize of 4 for each of the
% six conditions. This is analagous to those in Davis's/Ionatan's code.
stimulus.stair0{1} = upDownStaircase(1,1,threshStair0a,stepsize,1);
stimulus.stair0{1}.minThreshold = 0;
stimulus.stair0{2} = upDownStaircase(1,1,threshStair0b,stepsize,1);
stimulus.stair0{2}.minThreshold = 0;
stimulus.stair1{1} = upDownStaircase(1,1,threshStair1a,stepsize,1);
stimulus.stair1{1}.minThreshold = 0;
stimulus.stair1{2} = upDownStaircase(1,1,threshStair1b,stepsize,1);
stimulus.stair1{2}.minThreshold = 0;
stimulus.stair2{1} = upDownStaircase(1,1,threshStair2a,stepsize,1);
stimulus.stair2{1}.minThreshold = 0;
stimulus.stair2{2} = upDownStaircase(1,1,threshStair2b,stepsize,1);
stimulus.stair2{2}.minThreshold = 0;


% Initialize output arrays for each each rotation speed x eye movement direction = 6 conditions
stimulus.totalResponse = [];
stimulus.responseLlo = []; stimulus.responseLhi = []; stimulus.responseRlo = [];  stimulus.responseRhi = []; stimulus.responsePT = [];

% Initialize output arrays for each each rotation speed x eye movement direction = 6 conditions
stimulus.headingLlo = []; stimulus.headingLhi = []; stimulus.headingRlo = []; stimulus.headingRhi = []; stimulus.headingPT = [];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize important variables %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

stimulus.plane_dist                      = [12.5 25];                            % m
stimulus.dot_density                     = [.16 .16];                            % dots/deg^2 on screen
if stimulus.whichSetup == 1
    stimulus.view_window                 = [40 32];
else
    stimulus.view_window                 = [55 32];
end
stimulus.exclude                         = [-20 -3 20 3];                        % Don't place elements where the FOE will be

stimulus.padding = 15; % deg % how many deg to pad initial frame plane with so when plane moves over time, you don't see the plane's edge. 
                            % attn: Set to 0 if you want to check the number of dots per plane at the beginning of trial. set to 15 otherwise

stimulus.rotateConditions                = [0 0 0];
stimulus.rotation = 0;


stimulus.translation_speed               = 1.5;                                  % m/s, apparently 1.9 m/s is a brisk walking speed % reduced to get rid of temporal aliasing
stimulus.trans_axes                      = [3 1];                                % which way are we driving? 1=X,2=Y,3=Z first number is main direction, second is judged direction
stimulus.rot_axis                        = [0 1 0];                              % X (pitch),Y (yaw),Z (roll)

%-----Rig Settings----------------------
stimulus.view_dist                       = .57;                                  % m, distance of observer's eye from the monitor
stimulus.nPlanes                         = length(stimulus.plane_dist);          % how many planes of dots?

stimulus.trialCounter = 0; % initialize trialCounter



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % run the eye calibration
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
myscreen = eyeCalibDisp(myscreen); % OPTIONAL

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start text
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

displayTxt = sprintf('Heading Left: ''1'' Key      Heading Right: ''2'' Key');
mglTextSet('Arial',32,[1 1 1],[],[],[],[],[],[],[]);
mglTextDraw(displayTxt,[0 -2.5]);
mglFlush;
mglTextSet('Arial',32,[1 1 1],[],[],[],[],[],[],[]);
mglTextDraw('Press backtick when you are ready to begin.',[0 -5]);
mglFlush;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main display loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phaseNum = 1;
while (phaseNum <= length(task)) && ~myscreen.userHitEsc
    % update the task
    [task myscreen phaseNum] = updateTask(task,myscreen,phaseNum);
    % flip screen
    myscreen = tickScreen(myscreen,task);
end

% Put stuff here that you want to happen at the end

% print out start thresholds for next run
disp(sprintf('\n% --------------------------------------------- %\n'));
disp(sprintf('OFexpt_EM(''threshStair0a=%0.2f'', ''threshStair0b=%0.2f'',''threshStair1a=%0.2f'',''threshStair1b=%0.2f'',''threshStair2a=%0.2f'',''threshStair2b=%0.2f'');', ...
    stimulus.stair0{1}.threshold, stimulus.stair0{2}.threshold,stimulus.stair1{1}.threshold, stimulus.stair1{2}.threshold,stimulus.stair2{1}.threshold, stimulus.stair2{2}.threshold));
disp(sprintf('\n% --------------------------------------------- %\n'));

% REMOVE HUGE FIELDS WHICH CAUSE FILE SIZE TO BLOW UP
fieldsToRemove = {'phases','angle1','angle2','new_dots_m','new_dots_deg'};
stimulus = rmfield(stimulus,fieldsToRemove);

if isfield(stimulus,'trialRT')
stimulus.avgRT = nanmean(stimulus.trialRT);
end

% If we got here, we are at the end of the experiment.
myscreen = endTask(myscreen,task);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function that gets called at the start of each segment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = startSegmentCallback(task, myscreen)
global stimulus;
global MGL;

mglClearScreen; % clear the screen first


if task.thistrial.thisseg == 1
    
    stimulus.phases = [];  stimulus.angle1 = [];  stimulus.angle2 = []; stimulus.dots_m = []; stimulus.dots_deg = []; stimulus.new_dots_m = []; stimulus.new_dots_deg = []; % empty these out because they are large...
    
    stimulus.trialCounter = stimulus.trialCounter + 1;
    stimulus.frameCounter = 0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% Set Staircases, Headings, and Rotations for this trial %%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Which staircase will be updated? Flips a coin.
    stimulus.whichStair0 = round(rand)+1;  stimulus.whichStair1 = round(rand)+1;  stimulus.whichStair2 = round(rand)+1; % pick a random staircase either 1 or 2 for each condition
    
    
    if rand()>.5 % randomly make heading direction negative or positive (left or right) on each trial
        task.thistrial.headingSpeedandDir0 = stimulus.stair0{stimulus.whichStair0}.threshold; % low speed rot % get current threshold for all three conditions from the staircases
        task.thistrial.headingSpeedandDir1 = stimulus.stair1{stimulus.whichStair1}.threshold; % high speed rot
        task.thistrial.headingSpeedandDir2 = stimulus.stair2{stimulus.whichStair2}.threshold; % no rotation, pure translation
    else
        task.thistrial.headingSpeedandDir0 = -stimulus.stair0{stimulus.whichStair0}.threshold; % get current threshold for all three conditions from the staircases
        task.thistrial.headingSpeedandDir1 = -stimulus.stair1{stimulus.whichStair1}.threshold;
        task.thistrial.headingSpeedandDir2 = -stimulus.stair2{stimulus.whichStair2}.threshold;
    end
    
    
    % set all to 0 to train on pure translation.
    if stimulus.trialType(stimulus.trialCounter) == 1
        stimulus.rotateConditions = [0 -.8 0];
    elseif stimulus.trialType(stimulus.trialCounter) == 2
        stimulus.rotateConditions = [0 -2 0];
    elseif stimulus.trialType(stimulus.trialCounter) == 3
        stimulus.rotateConditions = [0 .8 0];
    elseif stimulus.trialType(stimulus.trialCounter) == 4
        stimulus.rotateConditions = [0 2 0];
    elseif stimulus.trialType(stimulus.trialCounter) == 5
        stimulus.rotateConditions = [0 0 0];
    end
    
    %{
        % set all to 0 to train on pure translation.
    if stimulus.trialType(stimulus.trialCounter) == 1
        stimulus.rotateConditions = [0 0 0];
    elseif stimulus.trialType(stimulus.trialCounter) == 2
        stimulus.rotateConditions = [0 0 0];
    elseif stimulus.trialType(stimulus.trialCounter) == 3
        stimulus.rotateConditions = [0 0 0];
    elseif stimulus.trialType(stimulus.trialCounter) == 4
        stimulus.rotateConditions = [0 0 0];
    elseif stimulus.trialType(stimulus.trialCounter) == 5
        stimulus.rotateConditions = [0 0 0];
    end
    %}
    
    stimulus.translate = [0 0 0]; % initalize translation vector
    
    % set heading direction for the trial based on condition and staircase
    if stimulus.trialType(stimulus.trialCounter) == 1 || stimulus.trialType(stimulus.trialCounter) == 3
        stimulus.translate(stimulus.trans_axes(1)) = stimulus.translation_speed*cosd(task.thistrial.headingSpeedandDir0);
        stimulus.translate(stimulus.trans_axes(2)) = stimulus.translation_speed*sind(task.thistrial.headingSpeedandDir0);
    elseif stimulus.trialType(stimulus.trialCounter) == 2 || stimulus.trialType(stimulus.trialCounter) == 4
        stimulus.translate(stimulus.trans_axes(1)) = stimulus.translation_speed*cosd(task.thistrial.headingSpeedandDir1);
        stimulus.translate(stimulus.trans_axes(2)) = stimulus.translation_speed*sind(task.thistrial.headingSpeedandDir1);
    elseif stimulus.trialType(stimulus.trialCounter) == 5
        stimulus.translate(stimulus.trans_axes(1)) = stimulus.translation_speed*cosd(task.thistrial.headingSpeedandDir2);
        stimulus.translate(stimulus.trans_axes(2)) = stimulus.translation_speed*sind(task.thistrial.headingSpeedandDir2);
    end
    
    
    if stimulus.feedback == 1 % Print heading and rotation if in feedback/debug
        
        
        disp(stimulus.trialCounter)
        if stimulus.trialType(stimulus.trialCounter) == 1
            disp('Low Speed Rotation Left')
            disp(['Heading = ' num2str(task.thistrial.headingSpeedandDir0)]);
        elseif stimulus.trialType(stimulus.trialCounter) == 3
            disp('Low Speed Rotation Right')
            disp(['Heading = ' num2str(task.thistrial.headingSpeedandDir0)]);
        elseif stimulus.trialType(stimulus.trialCounter) == 2
            disp('High Speed Rotation Left')
            disp(['Heading = ' num2str(task.thistrial.headingSpeedandDir1)]);
        elseif stimulus.trialType(stimulus.trialCounter) == 4
            disp('High Speed Rotation Right')
            disp(['Heading = ' num2str(task.thistrial.headingSpeedandDir1)]);
        else
            disp('Pure Translation')
            disp(['Heading = ' num2str(task.thistrial.headingSpeedandDir2)]);
        end
        
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Now make the stimulus %%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    for i_plane=1:stimulus.nPlanes
        
        % this code below equalizes the number of dots per plane in each
        % rotation condition
        if i_plane == 1
            if stimulus.rotateConditions(2) == 2 || stimulus.rotateConditions(2) == -2
                [stimulus.dots_m, stimulus.dots_deg] = make_dot_plane(1.9, stimulus.plane_dist(i_plane), [160 32],  [0 0 0 0]);
            elseif stimulus.rotateConditions(2) == .8 || stimulus.rotateConditions(2) == -.8
                [stimulus.dots_m, stimulus.dots_deg] = make_dot_plane(12.1, stimulus.plane_dist(i_plane), [170 32], [0 0 0 0]);
            elseif stimulus.rotateConditions(2) == 0
                [stimulus.dots_m, stimulus.dots_deg]= make_dot_plane(stimulus.dot_density(i_plane), stimulus.plane_dist(i_plane), stimulus.view_window, stimulus.exclude); % The planes should be infinite width for this to work bc otherwise they are offscreen (when observer path radius is larger than plane distance in Z, ie. R:T ratio is low) ... and then at the end trim the field of view using xlim([-20 20]); ylim([-16 16])
            end 
        elseif i_plane == 2
             if stimulus.rotateConditions(2) == 2 || stimulus.rotateConditions(2) == -2
                [stimulus.dots_m, stimulus.dots_deg] = make_dot_plane(.6, stimulus.plane_dist(i_plane), [160 32],  [0 0 0 0]);
            elseif stimulus.rotateConditions(2) == .8 || stimulus.rotateConditions(2) == -.8
                [stimulus.dots_m, stimulus.dots_deg] = make_dot_plane(2.85, stimulus.plane_dist(i_plane), [170 32], [0 0 0 0]);
            elseif stimulus.rotateConditions(2) == 0
                [stimulus.dots_m, stimulus.dots_deg]= make_dot_plane(stimulus.dot_density(i_plane), stimulus.plane_dist(i_plane), stimulus.view_window, stimulus.exclude); % The planes should be infinite width for this to work bc otherwise they are offscreen (when observer path radius is larger than plane distance in Z, ie. R:T ratio is low) ... and then at the end trim the field of view using xlim([-20 20]); ylim([-16 16])
            end
        end
        
        for i_frame=1:stimulus.mv_length
            stimulus.new_dots_m = update_dot_world_circular(stimulus.dots_m, stimulus.translate, stimulus.rotateConditions, 1/stimulus.frame_rate, (i_frame)); % starting from frame 1, rather than frame 0 bc circular path update dots code otherwise breaks when there is non-zero rotation, and creates very different projections for frame 0 and 1, which breaks the code below which throws out dots based on frame 1 only
            stimulus.new_dots_deg{i_plane,i_frame} = project_dot_world2(stimulus.new_dots_m, stimulus.view_window, stimulus.view_dist);
            
            % Must find the dots on image plane that are greater than the
            % screen size, and throw out the 3d points corresponding to
            % them, so the code is fast (otherwise must store thousands of
            % offscreen textures.  Same for exclusion region.
            
            if i_frame == 1
                
                % uncomment to see how many dots are really onScreen at
                % beginning of trial, rather than stimulus.padding at edges so we
                % don't ever see edges of planes (they are supposed to be
                % infinitely long  .      
                %throwAwayInds = stimulus.new_dots_deg{i_plane,i_frame}(1,:) > stimulus.view_window(1)/2 | stimulus.new_dots_deg{i_plane,i_frame}(1,:) < -stimulus.view_window(1)/2 | stimulus.new_dots_deg{i_plane,i_frame}(2,:) > stimulus.view_window(2)/2 | stimulus.new_dots_deg{i_plane,i_frame}(2,:) < -stimulus.view_window(2)/2;
               
                throwAwayInds = stimulus.new_dots_deg{i_plane,i_frame}(1,:) > stimulus.padding + stimulus.view_window(1)/2 | stimulus.new_dots_deg{i_plane,i_frame}(1,:) < -stimulus.padding + -stimulus.view_window(1)/2 | stimulus.new_dots_deg{i_plane,i_frame}(2,:) > stimulus.padding + stimulus.view_window(2)/2 | stimulus.new_dots_deg{i_plane,i_frame}(2,:) < -stimulus.padding + -stimulus.view_window(2)/2;

                % now do the same but for exclusion region
                exclude = stimulus.new_dots_deg{i_plane,i_frame}(1,:) < stimulus.exclude(3) & stimulus.new_dots_deg{i_plane,i_frame}(1,:) > stimulus.exclude(1) & stimulus.new_dots_deg{i_plane,i_frame}(2,:) < stimulus.exclude(4) & stimulus.new_dots_deg{i_plane,i_frame}(2,:) > stimulus.exclude(2);
                
                throwAwayInds2 = throwAwayInds | exclude; % combine
                
                stimulus.new_dots_deg{i_plane,i_frame}(:,throwAwayInds2) = []; % throwaway offscreen dots bc otherwise it will be slow.
                %keyboard   % uncomment to check number of dots per plane, but remember to set stimulus.padding to 0 first
                stimulus.dots_m(:,throwAwayInds2) = []; % need to throw away dots in 3d corresponding to 2d dots offscreen. 
            end 
            
            if i_frame == 1 || i_frame == 2
                %keyboard
                % debug
            end

        end
        
        stimulus.phases{i_plane} = ceil(rand(1,size(stimulus.new_dots_deg{i_plane,1},2))*360);
        
        stimulus.angle1{i_plane}  = ceil(rand(1,size(stimulus.new_dots_deg{i_plane,1},2))*180); % select a random angle that the first of 2 orthogonal gratings will be at in the envelope motion condition.
        %stimulus.angle2{i_plane} = wrapTo180(stimulus.angle1{i_plane} - 90); % select the orthogonal angle
               
    end
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function that gets called to draw the stimulus each frame
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = screenUpdateCallback(task, myscreen)

global MGL;
global stimulus;
mglClearScreen;

if task.thistrial.thisseg == 2 % Segment 2: Reponse
    mglClearScreen; % show a blank screen during response period
    stimulus.xpos_fixation = 0; % reset fixation variable
    if stimulus.whichSetup == 0
        mglGluDisk(stimulus.xpos_fixation, 0, .03,  [1 0 0], 24, 2); % present fixation during response period
    else
        mglGluDisk(stimulus.xpos_fixation, 0, .1,  [1 0 0], 24, 2); % present fixation during response period
    end
end

if task.thistrial.thisseg == 1 % Segment 1: Stimulus Presentation

    
    stimulus.frameCounter = stimulus.frameCounter + 1; % count which frame we're presenting
    
    if stimulus.frameCounter <= stimulus.mv_length 
        
        % nPlane 2   % plane 2 should be behind plane 1, so plot it first
        mglBltTexture(stimulus.tex(stimulus.phases{2}), stimulus.new_dots_deg{2,stimulus.frameCounter}',0,0, stimulus.angle1{2});

        % nPlane 1
        mglBltTexture(stimulus.tex(stimulus.phases{1}), stimulus.new_dots_deg{1,stimulus.frameCounter}',0,0, stimulus.angle1{1});

    end
    
    stimulus.xpos_fixation = 0;
    if stimulus.whichSetup == 0
        mglGluDisk(stimulus.xpos_fixation, 0, .03,  [1 0 0], 24, 2); % draw fixation dot
    else
        mglGluDisk(stimulus.xpos_fixation, 0, .1,  [1 0 0], 24, 1); % draw fixation dot
    end
   
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function that gets subject response
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = responseCallback(task, myscreen)
global stimulus;

stimulus.totalResponse(stimulus.trialCounter) = task.thistrial.whichButton;


%make sure we have not already received a response
if task.thistrial.gotResponse==0
    
    stimulus.trialRT(stimulus.trialCounter) = task.thistrial.reactionTime;
    
    if stimulus.trialType(stimulus.trialCounter) == 1 || stimulus.trialType(stimulus.trialCounter) == 3
        if sign(task.thistrial.headingSpeedandDir0) == -1; recodedHeading=1; elseif sign(task.thistrial.headingSpeedandDir0) == 1; recodedHeading=2;else recodedHeading=NaN;end % have to recode negative dot motion directions to rightward heading directions and vice versa for each condition!
        
        if recodedHeading == task.thistrial.whichButton
            %play the correct sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Tink'); end
            %update staircase
            stimulus.stair0{stimulus.whichStair0} = upDownStaircase(stimulus.stair0{stimulus.whichStair0}, 1);
        elseif isnan(recodedHeading)
            % if the stimulus is directly at the discrimination boundary,
            % flip a coin for which tone to play and update staircase
            if stimulus.feedback == 1
                if rand() >.5
                    mglPlaySound('Pop');
                else
                    mglPlaySound('Tink');
                end
            end
            stimulus.stair0{stimulus.whichStair0} = upDownStaircase(stimulus.stair0{stimulus.whichStair0}, 0);
        else
            %play the incorrect sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Pop'); end
            %update staircase
            stimulus.stair0{stimulus.whichStair0} = upDownStaircase(stimulus.stair0{stimulus.whichStair0}, 0);
        end
        
    elseif stimulus.trialType(stimulus.trialCounter) == 2 || stimulus.trialType(stimulus.trialCounter) == 4
        if sign(task.thistrial.headingSpeedandDir1) == -1; recodedHeading=1; elseif sign(task.thistrial.headingSpeedandDir1) == 1; recodedHeading=2;else recodedHeading=NaN;end % have to recode negative dot motion directions to rightward heading directions and vice versa for each condition!
        
        if recodedHeading == task.thistrial.whichButton
            %play the correct sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Tink'); end
            %update staircase
            stimulus.stair1{stimulus.whichStair1} = upDownStaircase(stimulus.stair1{stimulus.whichStair1}, 1);
        elseif isnan(recodedHeading)
            % if the stimulus is directly at the discrimination boundary,
            % flip a coin for which tone to play and update staircase
            if stimulus.feedback == 1
                if rand() >.5
                    mglPlaySound('Pop');
                else
                    mglPlaySound('Tink');
                end
            end
            stimulus.stair1{stimulus.whichStair1} = upDownStaircase(stimulus.stair1{stimulus.whichStair1}, 0);
        else
            %play the incorrect sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Pop'); end
            %update staircase
            stimulus.stair1{stimulus.whichStair1} = upDownStaircase(stimulus.stair1{stimulus.whichStair1}, 0);
        end
        
    elseif stimulus.trialType(stimulus.trialCounter) == 5
        if sign(task.thistrial.headingSpeedandDir2) == -1; recodedHeading=1; elseif sign(task.thistrial.headingSpeedandDir2) == 1; recodedHeading=2;else recodedHeading=NaN;end % have to recode negative dot motion directions to rightward heading directions and vice versa for each condition!
        
        if recodedHeading == task.thistrial.whichButton
            %play the correct sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Tink'); end
            %update staircase
            stimulus.stair2{stimulus.whichStair2} = upDownStaircase(stimulus.stair2{stimulus.whichStair2}, 1);
                elseif isnan(recodedHeading)
            % if the stimulus is directly at the discrimination boundary,
            % flip a coin for which tone to play and update staircase
            if stimulus.feedback == 1
                if rand() >.5
                    mglPlaySound('Pop');
                else
                    mglPlaySound('Tink');
                end
            end
            stimulus.stair2{stimulus.whichStair2} = upDownStaircase(stimulus.stair2{stimulus.whichStair2}, 0);
        else
            %play the incorrect sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Pop'); end
            %update staircase
            stimulus.stair2{stimulus.whichStair2} = upDownStaircase(stimulus.stair2{stimulus.whichStair2}, 0);
        end
        
    end
    
    % Now output responses and headings in appropriate arrays
    if stimulus.trialType(stimulus.trialCounter) == 1
        stimulus.responseLlo = [stimulus.responseLlo task.thistrial.whichButton];
        stimulus.headingLlo = [stimulus.headingLlo task.thistrial.headingSpeedandDir0];
    elseif stimulus.trialType(stimulus.trialCounter) == 2
        stimulus.responseLhi = [stimulus.responseLhi task.thistrial.whichButton];
        stimulus.headingLhi = [stimulus.headingLhi task.thistrial.headingSpeedandDir1];
    elseif stimulus.trialType(stimulus.trialCounter) == 3
        stimulus.responseRlo = [stimulus.responseRlo task.thistrial.whichButton];
        stimulus.headingRlo = [stimulus.headingRlo task.thistrial.headingSpeedandDir0];
    elseif stimulus.trialType(stimulus.trialCounter) == 4
        stimulus.responseRhi = [stimulus.responseRhi task.thistrial.whichButton];
        stimulus.headingRhi = [stimulus.headingRhi task.thistrial.headingSpeedandDir1];
    elseif stimulus.trialType(stimulus.trialCounter) == 5
        stimulus.responsePT = [stimulus.responsePT task.thistrial.whichButton];
        stimulus.headingPT = [stimulus.headingPT task.thistrial.headingSpeedandDir2];
    end

end


function stimulus = myInitStimulus(stimulus, myscreen, task)

% store a variable that lists which of the segments we are presenting the stimulus in
stimulus.stimulusSegments = 1;

% which phases we will have
stimulus.phasesRange = 1:360;

% stimulus size properties (in deg)
stimulus.size = .3;
stimulus.transition = .2; 
stimulus.textureSize = .4;

% make the window for the raised cosine
xDeg2pix = mglGetParam('xDeviceToPixels');
yDeg2pix = mglGetParam('yDeviceToPixels');
widthPixels = round(stimulus.textureSize*xDeg2pix);
heightPixels = round(stimulus.textureSize*yDeg2pix);
widthPixels = widthPixels + mod(widthPixels+1,2);
heightPixels = heightPixels + mod(heightPixels+1,2);
% remember, x and y are flipped
disc = mkDisc([heightPixels widthPixels], ... % matrix size
    stimulus.size*xDeg2pix/2, ... % radius
    [heightPixels widthPixels]/2, ... % origin
    stimulus.transition * xDeg2pix); % transition

% spatial frequency & contrast
stimulus.sf = 3;

stimulus.contrast = 1; 

for i = 1:length(stimulus.phasesRange)
    
    % in this version of the code, I generate a plaid here, instead of
    % summing two gratings above when I plot
    
    angle0_90 = [0 90];
    for jj = 1:2
    % make a gabor patch
    grating = mglMakeGrating(stimulus.textureSize, stimulus.textureSize, stimulus.sf, angle0_90(jj), stimulus.phasesRange(i));
    % multiply by the contrast
    grating = grating*stimulus.contrast;
    % multiple by a raised cosine disc
    grating2{jj} = grating.*disc;
    end
    
    % scale it to be between 0 and 255
    plaid = 255*((grating2{1}/2+grating2{2}/2)+1)/2;

    % convert grayscale to RGBA and add alpha layer so we can blend two
    % layers of gabor to create composite motion
    plaid_RGB = zeros(size(plaid,1),size(plaid,2),4);
    for ii = 1:3
        plaid_RGB(:,:,ii) = plaid;
    end
    
    plaid_RGB(:,:,4) = 255;
    
    % make it into a texture
    stimulus.tex(i) = mglCreateTexture(plaid_RGB);
    
end

