function [stimulus] = OFexpt_PM_Last_High_Density(varargin)
%      title: OFexpt_PM_Last_High_Density.m
%      usage: run with external monitor and install MGL toolbox first
%         by: Charlie S. Burlingham 
%    updated: November 23, 2020
%
%    purpose: generates non-varying phase motion (double plane, double patch density) stimulus video
%
%             Non-varying phase motion (last), two planes, just +-2 /s rotation, but double the patch density.
%
%             Trial Types:
%             1) Non-Varying Phase Motion, Simulated Eye Movement, Rotation Left, 2 /s
%             2) Non-Varying Phase Motion, Simulated Eye Movement, Rotation Right, 2 /s
%             3) Non-Varying Phase Motion, Pure Translation
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

mglClose; mglFlush;
getArgs(varargin,[],'verbose=1');
global MGL

%-----init some defaults for variables we can set in the function call ----------
if ieNotDefined('feedback'), feedback = 1; end % include tone feedback and debug text or no
if ieNotDefined('threshStair0a'), threshStair0a = 20; end % start at limits of -20 and 20, so you have some data for the asymptotes of psychometric curve
if ieNotDefined('threshStair0b'), threshStair0b = 20; end % initial staircase value, +-deg
if ieNotDefined('threshStair1a'), threshStair1a = 20; end
if ieNotDefined('threshStair1b'), threshStair1b = 20; end
if ieNotDefined('stepsize'), stepsize = 4; end  % initial stepsize of staircase, deg
if ieNotDefined('iterationCount'), iterationCount = 1; end
if ieNotDefined('numberOfRuns'), numberOfRuns = 11; end % 10 runs

%-----init the task----------
mglSetParam('mustSetSID',false,2); % don't require MGL subject ID database
myscreen.background = [127 127 127]; % set background color to mid-grey
myscreen.autoCloseScreen = 0;
myscreen.saveData = 1; % autosave data
myscreen = initScreen(myscreen);

task{1}.waitForBacktick = 0;
task{1}.seglen = [4 1]; % two segments: 1st displays stimulus (4 sec), 2nd is response interval (1 sec)
task{1}.getResponse = [0 1]; % get response in second interval
task{1}.random = 1;
task{1}.numTrials = 60; % must be divisible by 6 (num of trial conditions) so trial permutation works
% 60 trials * 5 seconds = 300 sec or 5 minutes

for phaseNum = 1:length(task)
    [task{phaseNum} myscreen] = initTask(task{phaseNum},myscreen,@startSegmentCallback,@screenUpdateCallback,@responseCallback);
end

%-----init the stimulus----------
global stimulus;
myscreen = initStimulus('stimulus',myscreen);
stimulus.feedback = feedback; % feedback on or off. determines tone feedback and debug output
stimulus = myInitStimulus(stimulus,myscreen,task);

%-----Set up trial condition permutation structure-------
stimulus.n_conditions = 3;
stimulus.trialType = [];

for ii = 1:stimulus.n_conditions
    stimulus.trialType = [stimulus.trialType ii*ones((task{1}.numTrials/stimulus.n_conditions),1)']; % populate vector with equal proportions of 0's, 1's, 2's, etc., which represent ALL trial conditions
end

stimulus.trialType = stimulus.trialType(randperm(task{1}.numTrials));   % randomly permutate order to randomize trial types in the run

%-----Screen Landmarks, Counters, and Stimulus Duration------------
stimulus_duration                        = 4;                           % secs
stimulus.duration                        = stimulus_duration;
frame_rate                               = 75;                          % Screen frame rate (hz) of the CRT
stimulus.frame_rate                      = frame_rate;
stimulus.mv_length              = ceil(stimulus_duration*frame_rate);   % length of stimulus movie
stimulus.plane_dist                      = [12.5 25];                   % m.
stimulus.dot_density                     = [.16 .16].*2;                % dots/deg^2 on screen. % Note: double density!
stimulus.view_window                     = [40 32];                     % CRT dimensions
stimulus.exclude                         = [-15 -2.25 15 2.25];         % deg. exclusion region, i.e., don't place elements where the FOE will be.
stimulus.nFlowFields                     = 10;
stimulus.trialCounter                    = 0;

%-----Staircase Settings------------
% Init FOUR 1 down 1 up staircase with a stepsize of 4 for each of the
% six conditions. This is analagous to those in Davis's/Ionatan's code.
stimulus.stair0{1} = upDownStaircase(1,1,threshStair0a,stepsize,1);
stimulus.stair0{1}.minThreshold = 0;
stimulus.stair0{2} = upDownStaircase(1,1,threshStair0b,stepsize,1);
stimulus.stair0{2}.minThreshold = 0;
stimulus.stair1{1} = upDownStaircase(1,1,threshStair1a,stepsize,1);
stimulus.stair1{1}.minThreshold = 0;
stimulus.stair1{2} = upDownStaircase(1,1,threshStair1b,stepsize,1);
stimulus.stair1{2}.minThreshold = 0;

% Initialize output arrays for each each rotation speed x eye movement direction = 3 conditions
stimulus.totalResponse = [];
stimulus.responseLhi = []; stimulus.responseRhi = []; stimulus.responsePT = [];

% Initialize output arrays for each each rotation speed x eye movement direction = 3 conditions
stimulus.headingLhi = []; stimulus.headingRhi = []; stimulus.headingPT = [];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize important variables %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
stimulus = rmfield(stimulus,'phases');

stimulus.rotateConditions                = [0 0 0];
stimulus.translation_speed               = 1.5;                                  % m/s, apparently 1.9 m/s is a brisk walking speed 
stimulus.trans_axes                      = [3 1];                                % which way are we driving? 1=X,2=Y,3=Z first number is main direction, second is judged direction
stimulus.rot_axis                        = [0 1 0];                              % X (pitch),Y (yaw),Z (roll)

%-----Rig Settings----------------------
stimulus.view_dist                       = .57;                                  % m

stimulus.nPlanes                         = length(stimulus.plane_dist);          % 2 depth planes!

stimulus.trialCounter = 0; % initialize trialCounter


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Eye calibration
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
myscreen = eyeCalibDisp(myscreen); % OPTIONAL

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start text on screen
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

displayTxt = sprintf('Heading Left: ''1'' Key      Heading Right: ''2'' Key');
mglTextSet('Arial',32,[1 1 1],[],[],[],[],[],[],[]);
mglTextDraw(displayTxt,[0 -2.5]);
mglFlush;
mglTextSet('Arial',32,[1 1 1],[],[],[],[],[],[],[]);
mglTextDraw('Press backtick when you are ready to begin.',[0 -5]);
mglFlush;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main display loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phaseNum = 1;
while (phaseNum <= length(task)) && ~myscreen.userHitEsc
    % update the task
    [task myscreen phaseNum] = updateTask(task,myscreen,phaseNum);
    % flip screen
    myscreen = tickScreen(myscreen,task);
end

% REMOVE HUGE FIELDS WHICH CAUSE FILE SIZE TO BLOW UP
fieldsToRemove = {'phaseMovie_ver','phaseMovie_hor','dots_m','dots_deg'};
stimulus = rmfield(stimulus,fieldsToRemove);

% If we got here, we are at the end of the experiment.
myscreen = endTask(myscreen,task);

% Put stuff here that you want to happen at the end. start next run and
% carry over staircase values
if iterationCount < numberOfRuns
    iterationCount = iterationCount + 1; 
    threshStair0aStr = sprintf('threshStair0a=%0.2f',stimulus.stair0{1}.threshold);
    threshStair0bStr = sprintf('threshStair0b=%0.2f',stimulus.stair0{2}.threshold);
    threshStair1aStr = sprintf('threshStair1a=%0.2f',stimulus.stair1{1}.threshold);
    threshStair1bStr = sprintf('threshStair1b=%0.2f',stimulus.stair1{2}.threshold);
    iterationCountStr = sprintf('iterationCount=%0.2f',iterationCount);
    
    newstepsize = stepsize-1; 
    newstepsizetext = sprintf('stepsize=%0.2f',newstepsize);
    
    if mod(iterationCount,4) ~= 0

        % print out start thresholds for next run
        disp(sprintf('\n% --------------------------------------------- %\n'));
        disp(sprintf('OFexpt_PM_Last_High_Density(''threshStair0a=%0.2f'', ''threshStair0b=%0.2f'',''threshStair1a=%0.2f'',''threshStair1b=%0.2f'');', ...
            stimulus.stair0{1}.threshold, stimulus.stair0{2}.threshold,stimulus.stair1{1}.threshold, stimulus.stair1{2}.threshold));
        disp(sprintf('\n% --------------------------------------------- %\n'));
        
        OFexpt_PM_Last_High_Density(threshStair0aStr,threshStair0bStr,threshStair1aStr,threshStair1bStr,iterationCountStr,newstepsizetext);
        
        disp(sprintf('\n% --------------------------------------------- %\n'));
        
    elseif mod(iterationCount,4) == 0
        
        disp('starting from default params again')
        OFexpt_PM_Last_High_Density(['iterationCount=' num2str(iterationCount)]);

    end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function that gets called at the start of each segment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = startSegmentCallback(task, myscreen)
global stimulus;
global MGL;

mglClearScreen; % clear the screen first

if task.thistrial.thisseg == 1
        
    stimulus.trialCounter = stimulus.trialCounter + 1;
    stimulus.frameCounter = 0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% Set Staircases, Headings, and Rotations for this trial %%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Which staircase will be updated? Flips a coin.
    stimulus.whichStair0 = round(rand)+1;  stimulus.whichStair1 = round(rand)+1;  stimulus.whichStair2 = round(rand)+1; % pick a random staircase either 1 or 2 for each condition
    
    if rand()>.5 % randomly make heading direction negative or positive (left or right) on each trial
        task.thistrial.headingSpeedandDir0 = stimulus.stair0{stimulus.whichStair0}.threshold; % get current threshold for all three conditions from the staircases
        task.thistrial.headingSpeedandDir1 = stimulus.stair1{stimulus.whichStair1}.threshold;
    else
        task.thistrial.headingSpeedandDir0 = -stimulus.stair0{stimulus.whichStair0}.threshold; % get current threshold for all three conditions from the staircases
        task.thistrial.headingSpeedandDir1 = -stimulus.stair1{stimulus.whichStair1}.threshold;
    end
    
    if stimulus.trialType(stimulus.trialCounter) == 1
        stimulus.rotateConditions = [0 -2 0]; % left rotation 2/s
    elseif stimulus.trialType(stimulus.trialCounter) == 2
        stimulus.rotateConditions = [0 2 0]; % right rotation 2/s
    elseif stimulus.trialType(stimulus.trialCounter) == 3
        stimulus.rotateConditions = [0 0 0]; % no rotation
    end
    stimulus.translate = [0 0 0]; % initalize translation vector
    
    % set heading direction for the trial based on condition and staircase
    if stimulus.trialType(stimulus.trialCounter) == 1 || stimulus.trialType(stimulus.trialCounter) == 2
        stimulus.translate(stimulus.trans_axes(1)) = stimulus.translation_speed*cosd(task.thistrial.headingSpeedandDir1);
        stimulus.translate(stimulus.trans_axes(2)) = stimulus.translation_speed*sind(task.thistrial.headingSpeedandDir1);
    elseif stimulus.trialType(stimulus.trialCounter) == 3
        stimulus.translate(stimulus.trans_axes(1)) = stimulus.translation_speed*cosd(task.thistrial.headingSpeedandDir0);
        stimulus.translate(stimulus.trans_axes(2)) = stimulus.translation_speed*sind(task.thistrial.headingSpeedandDir0);
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Now make the stimulus %%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Init empty matrix of phases
    stimulus.phaseMovie_ver{1} = [];
    stimulus.phaseMovie_ver{2} = [];
    stimulus.phaseMovie_hor{1} = [];
    stimulus.phaseMovie_hor{2} = [];
    
    stimulus.dots_m = {};
    stimulus.dots_deg = {};
    
    if stimulus.feedback == 1 % Print heading and rotation if in feedback/debug
        
        
        disp(stimulus.trialCounter)
        
        if stimulus.trialType(stimulus.trialCounter) == 1
            
            if sign(task.thistrial.headingSpeedandDir1) == 1
                disp(['Heading Right:' num2str(task.thistrial.headingSpeedandDir1)])
            elseif sign(task.thistrial.headingSpeedandDir1) == -1
                disp(['Heading Left:' num2str(task.thistrial.headingSpeedandDir1)])
            else
                disp('Straight Ahead')
            end
            
            disp('2 /s Left')
            
        elseif stimulus.trialType(stimulus.trialCounter) == 2
            if sign(task.thistrial.headingSpeedandDir1) == 1
                disp(['Heading Right:' num2str(task.thistrial.headingSpeedandDir1)])
            elseif sign(task.thistrial.headingSpeedandDir1) == -1
                disp(['Heading Left:' num2str(task.thistrial.headingSpeedandDir1)])
            else
                disp('Straight Ahead')
            end
            disp('2 /s Right')
            
        elseif stimulus.trialType(stimulus.trialCounter) == 3
            
            if sign(task.thistrial.headingSpeedandDir0) == 1
                disp(['Heading Right:' num2str(task.thistrial.headingSpeedandDir0)])
            elseif sign(task.thistrial.headingSpeedandDir0) == -1
                disp(['Heading Left:' num2str(task.thistrial.headingSpeedandDir0)])
            else
                disp('Straight Ahead')
            end
            disp('Pure Translation')
            
        end
    end
    
    for i=1:stimulus.nPlanes % loop over both depth planes, and generate optic flow
        
        for iFlowField = 0:stimulus.nFlowFields-1
            
            % generate 3-d environment - one plane made of dots
            if iFlowField == 0
                [stimulus.dots_m{i}, stimulus.dots_deg{i}] = make_dot_plane(stimulus.dot_density(i), stimulus.plane_dist(i), stimulus.view_window, stimulus.exclude);
            end
            
            % generate optic flow field (last flow field in sequence only)
            stimulus.velocity_field{i} = calculate_plane_flow_timestamps(stimulus.dots_m{i}, stimulus.dots_deg{i}, stimulus.translate, stimulus.rotateConditions.*pi./180, stimulus.view_dist, stimulus.nFlowFields-1, stimulus.plane_dist(i), stimulus.nFlowFields, stimulus.duration); 
            
            stimulus.nGabors(i) = size(stimulus.dots_deg{i},2);
            
            % optionally plot flow fields:
            %{
            if iFlowField == 3 
            figure
            quiver(stimulus.dots_deg{i}(1,:),stimulus.dots_deg{i}(2,:),stimulus.velocity_field{i}(1,:),stimulus.velocity_field{i}(2,:));
            keyboard
            end
            %}
            
            % Convert image velocity into phase velocity to control windowed gratings comprising stimulus
            stimulus.speed_verT = stimulus.velocity_field{i}(2,:);
            stimulus.speed_horT = stimulus.velocity_field{i}(1,:);
            
            stimulus.angle_ver{i} = 270*(stimulus.speed_verT>=0)+90*(stimulus.speed_verT<0); % in deg
            stimulus.angle_hor{i} = 180*(stimulus.speed_horT>0); % in deg
            
            stimulus.angle_ver{i} = rad2deg(deg2rad(stimulus.angle_ver{i}) + deg2rad(90));
            stimulus.angle_hor{i} = rad2deg(deg2rad(stimulus.angle_hor{i}) + deg2rad(90));
            
            stimulus.speed_ver{i} = abs(stimulus.velocity_field{i}(2,:));
            stimulus.speed_hor{i} = abs(stimulus.velocity_field{i}(1,:));
            
            % main calculation. image velocity -> phase velocity
            stimulus.phase_step_ver{i} = round(stimulus.speed_ver{i}*(stimulus.sf/stimulus.duration)*360/stimulus.frame_rate)';
            stimulus.phase_step_hor{i} = round(stimulus.speed_hor{i}*(stimulus.sf/stimulus.duration)*360/stimulus.frame_rate)';
            
            if iFlowField == 0
                % set a random phase per grating for first flow field in sequence
                stimulus.phases_verT = repmat(ceil(rand(size(stimulus.speed_ver{i},2),1)*360),1,stimulus.mv_length/stimulus.nFlowFields)+repmat(stimulus.phase_step_ver{i},1,stimulus.mv_length/stimulus.nFlowFields).*repmat(0:(stimulus.mv_length/stimulus.nFlowFields-1),size(stimulus.speed_ver{i},2),1);
                stimulus.phases_verT = rem(stimulus.phases_verT,360);
                stimulus.phases_ver{i} = stimulus.phases_verT + (stimulus.phases_verT==0)*360;
                
                stimulus.phases_horT = repmat(ceil(rand(size(stimulus.speed_hor{i},2),1)*360),1,stimulus.mv_length/stimulus.nFlowFields)+repmat(stimulus.phase_step_hor{i},1,stimulus.mv_length/stimulus.nFlowFields).*repmat(0:(stimulus.mv_length/stimulus.nFlowFields-1),size(stimulus.speed_hor{i},2),1);
                stimulus.phases_horT = rem(stimulus.phases_horT,360);
                stimulus.phases_hor{i} = stimulus.phases_horT + (stimulus.phases_horT==0)*360;
            else
                stimulus.phases_verT = repmat(stimulus.phases_ver{i}(:,end),1,stimulus.mv_length/stimulus.nFlowFields)+repmat(stimulus.phase_step_ver{i},1,stimulus.mv_length/stimulus.nFlowFields).*repmat(0:(stimulus.mv_length/stimulus.nFlowFields-1),size(stimulus.speed_ver{i},2),1);
                stimulus.phases_verT = rem(stimulus.phases_verT,360);
                stimulus.phases_ver{i} = stimulus.phases_verT + (stimulus.phases_verT==0)*360;
                
                stimulus.phases_horT = repmat(stimulus.phases_hor{i}(:,end),1,stimulus.mv_length/stimulus.nFlowFields)+repmat(stimulus.phase_step_hor{i},1,stimulus.mv_length/stimulus.nFlowFields).*repmat(0:(stimulus.mv_length/stimulus.nFlowFields-1),size(stimulus.speed_hor{i},2),1);
                stimulus.phases_horT = rem(stimulus.phases_horT,360);
                stimulus.phases_hor{i} = stimulus.phases_horT + (stimulus.phases_horT==0)*360;
            end
            
            % phase movies capture phase velocity over time (non-varying or time-varying, depending on condition)
            stimulus.phaseMovie_ver{i} = cat(2, stimulus.phaseMovie_ver{i}, stimulus.phases_ver{i});
            stimulus.phaseMovie_hor{i} = cat(2, stimulus.phaseMovie_hor{i}, stimulus.phases_hor{i});     
        end
        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function that gets called to draw the stimulus each frame
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = screenUpdateCallback(task, myscreen)

global MGL;
global stimulus;
mglClearScreen;

if task.thistrial.thisseg == 2 % Segment 2: Reponse
     mglClearScreen; % show a blank screen during response period
     stimulus.xpos_fixation = 0; % reset fixation variable   
     mglGluDisk(stimulus.xpos_fixation, 0, .09,  [1 0 0], 60, 20); % present fixation during response period
end

if task.thistrial.thisseg == 1 % Segment 1: Stimulus Presentation
        
    stimulus.frameCounter = stimulus.frameCounter + 1; % count which frame we're presenting
    
    % actually draw the stimulus (many windowed gratings) on the screen
    if stimulus.frameCounter <= stimulus.mv_length 
        % nPlane 2
        mglBltTexture(stimulus.tex(stimulus.phaseMovie_hor{2}(:, stimulus.frameCounter)), stimulus.dots_deg{2}',0,0, stimulus.angle_hor{2});
        mglBltTexture(stimulus.tex(stimulus.phaseMovie_ver{2}(:, stimulus.frameCounter)), stimulus.dots_deg{2}',0,0, stimulus.angle_ver{2});
        
        % nPlane 1
        mglBltTexture(stimulus.tex(stimulus.phaseMovie_hor{1}(:, stimulus.frameCounter)), stimulus.dots_deg{1}',0,0, stimulus.angle_hor{1});
        mglBltTexture(stimulus.tex(stimulus.phaseMovie_ver{1}(:, stimulus.frameCounter)), stimulus.dots_deg{1}',0,0, stimulus.angle_ver{1}); 
    end
    
    mglGluDisk(0, 0, .09,  [1 0 0], 60, 20); % draw fixation cross

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function that gets subject response
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [task myscreen] = responseCallback(task, myscreen)
global stimulus;

stimulus.totalResponse(stimulus.trialCounter) = task.thistrial.whichButton;

% make sure we have not already received a response
if task.thistrial.gotResponse==0
    
    stimulus.trialRT(stimulus.trialCounter) = task.thistrial.reactionTime;
    
    if stimulus.trialType(stimulus.trialCounter) == 3
        if sign(task.thistrial.headingSpeedandDir0) == -1; recodedHeading=1; elseif sign(task.thistrial.headingSpeedandDir0) == 1; recodedHeading=2;else recodedHeading=NaN;end 
        
        if recodedHeading == task.thistrial.whichButton
            %play the correct sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Tink'); end
            %update staircase
            stimulus.stair0{stimulus.whichStair0} = upDownStaircase(stimulus.stair0{stimulus.whichStair0}, 1);
        elseif isnan(recodedHeading)
            % if the stimulus is directly at the discrimination boundary,
            % flip a coin for which tone to play and update staircase
            if stimulus.feedback == 1
                if rand() >.5
                    mglPlaySound('Pop');
                else
                    mglPlaySound('Tink');
                end
            end
            stimulus.stair0{stimulus.whichStair0} = upDownStaircase(stimulus.stair0{stimulus.whichStair0}, 0);
        else
            %play the incorrect sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Pop'); end
            %update staircase
            stimulus.stair0{stimulus.whichStair0} = upDownStaircase(stimulus.stair0{stimulus.whichStair0}, 0);
        end
        
    elseif stimulus.trialType(stimulus.trialCounter) == 1 || stimulus.trialType(stimulus.trialCounter) == 2
        
        if sign(task.thistrial.headingSpeedandDir1) == -1; recodedHeading=1; elseif sign(task.thistrial.headingSpeedandDir1) == 1; recodedHeading=2;else recodedHeading=NaN;end
        
        if recodedHeading == task.thistrial.whichButton
            %play the correct sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Tink'); end
            %update staircase
            stimulus.stair1{stimulus.whichStair1} = upDownStaircase(stimulus.stair1{stimulus.whichStair1}, 1);
        elseif isnan(recodedHeading)
            % if the stimulus is directly at the discrimination boundary,
            % flip a coin for which tone to play and update staircase
            if stimulus.feedback == 1
                if rand() >.5
                    mglPlaySound('Pop');
                else
                    mglPlaySound('Tink');
                end
            end
            stimulus.stair1{stimulus.whichStair1} = upDownStaircase(stimulus.stair1{stimulus.whichStair1}, 0);
        else
            %play the incorrect sound if feedback
            if stimulus.feedback == 1; mglPlaySound('Pop'); end
            %update staircase
            stimulus.stair1{stimulus.whichStair1} = upDownStaircase(stimulus.stair1{stimulus.whichStair1}, 0);
        end
        
    end
    
    % Now output responses and headings in appropriate arrays
    if stimulus.trialType(stimulus.trialCounter) == 1
        stimulus.responseLhi = [stimulus.responseLhi task.thistrial.whichButton];
        stimulus.headingLhi = [stimulus.headingLhi task.thistrial.headingSpeedandDir1];
    elseif stimulus.trialType(stimulus.trialCounter) == 2
        stimulus.responseRhi = [stimulus.responseRhi task.thistrial.whichButton];
        stimulus.headingRhi = [stimulus.headingRhi task.thistrial.headingSpeedandDir1];
    elseif stimulus.trialType(stimulus.trialCounter) == 3
        stimulus.responsePT = [stimulus.responsePT task.thistrial.whichButton];
        stimulus.headingPT = [stimulus.headingPT task.thistrial.headingSpeedandDir0];
    end
    
end


function stimulus = myInitStimulus(stimulus, myscreen, task)
% Specifications of each windowed grating on-screen

% store a variable that lists which of the segments we are presenting the stimulus in
stimulus.stimulusSegments = 1;

% which phases we will have
stimulus.phases = 1:360;

% stimulus size properties (in deg)
stimulus.size = .3;
stimulus.transition = .2; 
stimulus.textureSize = .4;

% make the window for the raised cosine
xDeg2pix = mglGetParam('xDeviceToPixels');
yDeg2pix = mglGetParam('yDeviceToPixels');
widthPixels = round(stimulus.textureSize*xDeg2pix);
heightPixels = round(stimulus.textureSize*yDeg2pix);
widthPixels = widthPixels + mod(widthPixels+1,2);
heightPixels = heightPixels + mod(heightPixels+1,2);
% remember, x and y are flipped
disc = mkDisc([heightPixels widthPixels], ... % matrix size
    stimulus.size*xDeg2pix/2, ... % radius
    [heightPixels widthPixels]/2, ... % origin
    stimulus.transition * xDeg2pix); % transition

% spatial frequency & contrast
stimulus.sf = 3; % cyc/deg

stimulus.contrast = 1; 

for i = 1:length(stimulus.phases)
    % make a gabor patch
    grating = mglMakeGrating(stimulus.textureSize, stimulus.textureSize, stimulus.sf, 0, stimulus.phases(i));
    % multiply by the contrast
    grating = grating*stimulus.contrast;
    % multiple by a raised cosine disc
    grating = grating.*disc;
    % scale it to be between 0 and 255
    grating = 255*(grating+1)/2;
    
    % convert grayscale to RGBA and add alpha layer so we can blend two
    % layers of gabor to create composite motion
    grating_RGBA = zeros(size(grating,1),size(grating,2),4);
    for ii = 1:3
        grating_RGBA(:,:,ii) = grating;
    end
    
    grating_RGBA(:,:,4) = 255/2; % half of full opacity
    
    ff = grating_RGBA(:,:,4);
    ff(grating_RGBA(:,:,3) == 255/2) = 0; % set the surround of the windowed grating to be transparent to prevent overlap.
    grating_RGBA(:,:,4) = ff;
      
    % make it into a texture
    stimulus.tex(i) = mglCreateTexture(grating_RGBA);
end

