function dots = update_dot_world_circular(dots, trans, rot, dt, nSteps)
%
% Author: Charlie S. Burlingham
% September 11, 2019. 
%
% This code simulates self-motion along a circular path, given a fronto-
% parallel plane. Implemented based on my own derivations, and 
% Stone and Perrone's (1997) methods.
%
% Takes the initial array of dots, and moves them to new positions in
% observer-centered coordinates. Right now the dots have infinite
% lifetimes, but eventually this would be the place to implement death and
% rebirth if we wanted. 
% 
% dots is a 3 row vector containing the [X,Y,Z] positions of the dots, in 
%       m.
% 
% trans is a vector of the observer's velocity [X,Y,Z] in m/s.
% 
% rot is a vector of the observer's rotation [pitch, yaw, roll] in deg/s 
%       (i.e. around the X, Y, and Z axes, respectively). Positive values
%       are counter-clockwise.
%
% dt is how long between refreshes of the world, in seconds. This could
%       theoretically be as fast as the refresh rate of the monitor, but
%       usually is less frequent.
%
% nSteps is how far into the movie we are. Each time we call this function,
%       we're starting with the initial world conditions to avoid any
%       unpleasantness from the observer rotating and translating at the
%       same time (the translation is fixed in world-centered coordinates,
%       but changes in viewer-centered, if the observer is rotating).
%
% dist is the distance of the plane to the observer in Z
% 
% dots the output is the new [X,Y,Z] position (in m) of all the dots,
%       viewer-centered.
%
% To debug, add keyboards before each transformations of "dots" and plot
% the 3d environment, using:
% figure(1); hold on; plot3(dots(1,:),dots(2,:),dots(3,:),'ko','markerEdgeColor','k','markerFaceColor','k'); view(0,0)
%
%
% Copyright (C) 2016-2020 Charlie S. Burlingham
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

% The radius of the circle = translation speed / sin(rotation speed)   [you can derive this on paper using trig]
transSpeed = sqrt(sum((trans).^2)); % speed along tangent of circle
r = transSpeed/sind(abs(rot(2))); % radius of the circle of the OBSERVER'S path over time


% Now compute the location of the observer relative to the circular path of
% the plane. 
headingAngle = sign(rot(2)).*atan(trans(1)/trans(3));  % This angle is with respect to left hand horizontal meridian (9:00 o'clock), like my diagrams, and what following trig requires.      
[coordsObserver_x coordsObserver_z] = pol2cart(deg2rad(180)-headingAngle,r); % now convert to be with respect to 3:00 o'clock, what pol2cart expects.

% The origin always starts 12.5 or 25 m from the plane, with the plan perpendicular to the line-of-sight.

% Now move the origin to the correct position for the orbiting (rotation),
% given a right triangle whose hypotenuse is the path circle's radius r, whose
% edges are the nodal point of the orbit, the edge of the path circle, and
% another edge so that it forms a right triangle.

if rot(2) ~=0 % only do this if you are rotating
    
    % Compute the lengths of the sides of this triangle:
    a = sqrt(r^2 - (r.*cos(deg2rad(90)-headingAngle)).^2);
    b = sqrt(r^2 - (r.*sin(deg2rad(90)-headingAngle)).^2);
    
    % Move the planes in the opposite direction side length b, and the
    % difference between the radius and side length a to get us from the
    % initial origin to the center of the orbit.
    if sign(rot(2)) == 1
        dots(1,:) = dots(1,:) + (r-a) ;
    elseif sign(rot(2)) == -1
        dots(1,:) = dots(1,:) - (r-a) ;
    end
    
    if sign(headingAngle) == 1 || headingAngle == 0
        dots(3,:) = dots(3,:) + b;
    elseif sign(headingAngle) == -1
        dots(3,:) = dots(3,:) - b;
    end
    
end

% Second, orbit the plane. We're going to do this axis by axis,
% because it makes our lives easier.

rotStep = -rot*dt*nSteps; % how much is plane rotating. this is equivalent to rotating plane as in the calculate_timstamps function for TVPM (CSB, aug 5 2019)

%These are rotation matrices, which move [X;Y;Z] positions to the new
%coordinate system c.f.:
% http://en.wikipedia.org/wiki/Rotation_matrix#In_three_dimensions

rotate_x = [1 0 0; 0 cosd(rotStep(1)) -sind(rotStep(1)); 0 sind(rotStep(1)) cosd(rotStep(1))];
rotate_y = [cosd(rotStep(2)) 0 sind(rotStep(2)); 0 1 0; -sind(rotStep(2)) 0 cosd(rotStep(2))];
rotate_z = [cosd(rotStep(3)) -sind(rotStep(3)) 0; sind(rotStep(3)) cosd(rotStep(3)) 0; 0 0 1];
rotation = rotate_x*rotate_y*rotate_z;
dots = rotation*dots; % orbit plane in opposite direction as observer rotation.

if rot(2) == 0 % if you're not rotating, translate plane at specified heading toward observer
    movement = (trans*dt*nSteps)';
else
    movement = [sign(rot(2))*coordsObserver_x 0 coordsObserver_z]'; % otherwise, now move the observer into the fixed location, simulating circular path. 
end

dots = dots-repmat(movement,1,size(dots,2)); % move from the plane orbit nodal point to the correct point on the observer's path. Move planes in negative direction, equiv to moving observers

